#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

enum time {
	TIME_MORNING,
	TIME_DAYTIME,
	TIME_NIGHT,
	TIME_UNKNOWN,
};

enum lang {
	LANG_ENGLISH,
	LANG_GERMAN,
};

static void usage(const char *prg)
{
	printf("usage: %s <-n|--name NAME> [-t|--time TIME] [-g|--german]\n", prg);
}

int main(int argc, char *argv[])
{
	int c;
	char *name = NULL;
	enum time time = TIME_UNKNOWN;
	enum lang lang = LANG_ENGLISH;
	char *greeting;

	while (1) {
		int option_index = 0;
		static struct option long_options[] = {
			/* name,        has_arg,           flag, val*/
			{"name",        required_argument, NULL, 'n'},
			{"time",        required_argument, NULL, 't'},
			{"german",      no_argument,       NULL, 'g'},
			{0,             0,                 0,    0},
		};

		c = getopt_long(argc, argv, "n:t:g",
				long_options, &option_index);
		if (c == -1)
			break;

		switch (c) {
		case 'n':
			name = strdup(optarg);
			if (name == NULL)
				exit(EXIT_FAILURE);
			break;
		case 't':
			if (strcmp(optarg, "morning") == 0)
				time = TIME_MORNING;
			else if (strcmp(optarg, "daytime") == 0)
				time = TIME_DAYTIME;
			else if (strcmp(optarg, "evening") == 0)
				time = TIME_NIGHT;
			else
				time = TIME_UNKNOWN;
			break;
		case 'g':
			lang = LANG_GERMAN;
			break;
		default:
			usage(argv[0]);
			exit(EXIT_SUCCESS);
		}
	}

	if (name == NULL) {
		/* NAME が指定されなかった */
		usage(argv[0]);
		return EXIT_FAILURE;
	}

	switch (time) {
	case TIME_MORNING:
		greeting =
			(lang == LANG_ENGLISH) ? "Good Morning" : "Guten Morgen";
		break;
	case TIME_DAYTIME:
		greeting =
			(lang == LANG_ENGLISH) ? "Good Afternoon" : "Guten Tag";
		break;
	case TIME_NIGHT:
		greeting =
			(lang == LANG_ENGLISH) ? "Good Night" : "Gute Nacht";
		break;
	default:
	case TIME_UNKNOWN:
		greeting =
			(lang == LANG_ENGLISH) ? "Hello" : "Hallo";
		break;
	}

	printf("%s, %s!\n", greeting, name);

	free(name);

	return EXIT_SUCCESS;
}
